#appel: entrop(f=hs,dim_x=3,dim_y=1,S=1000,n=100,R=1,LHS=FALSE,det=TRUE,paramlois=array(c(-pi,pi,-pi,pi,-pi,pi),dim=c(2,3)))

entrop<-function(f=a,modele=b,methode=0,dim_x,dim_y=1,S=10000,n=100,R=100,logx=rep(FALSE,dim_x),
	logy=rep(FALSE,dim_y),LHS=FALSE,det=FALSE,rel=FALSE,lois=rep(0,dim_x),
	paramlois=array(0,dim=c(4,dim_x)),indices1=rep(TRUE,dim_x),
	indices2=matrix(FALSE,nrow=dim_x,ncol=dim_x),affichage=FALSE) {

#-----------------------------------------------------------------------------------------------|
#Estimation des indices de sensibilite basee sur l'entropie
#-----------------------------------------------------------------------------------------------|
#Arguments obligatoires dans le cas d'un modele :
#		modele : objet contenant le modele
#		methode : numero de la methode, 0 si fonction utilisateur
#		dim_x : nombre de parametres d'entree dans le modele
#
#Arguments obligatoires dans le cas d'une fonction :
#		f : objet representant la fonction
#		dim_x : nombre de parametres d'entree dans le modele
#
#Arguments facultatifs :
#		dim_y: dimension de la sortie
#		S : nombre de simulations
#		n : nombre de sous-intervalles de variations des parametres
#		R : nombre de repetitions du programme
#		logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux
#			parametres d'entree transformees en log lors de
#			l'ajustement du modele
#		logy : mettre logy[i]=TRUE si la variable de sortie a ete
#			transformee en log lors de l'ajustement du modele
#		LHS : TRUE si on veut un echantillonage par hypercube latin
#		det: TRUE pour echantillonnage deterministe (OK pour uniforme)
#		rel : TRUE si normalisation par la somme des indices
#		lois : vecteur contenant les types de distribution de proba
#			pour chaque Vi
#			0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull
#			4=exponentielle ; 5=beta ; 6=triangulaire ; 7=trapeze
#			Par defaut, on prend la loi uniforme
#		paramlois : tableau contenant les parametres de chaque loi
#				pour chaque Vi (range par colonne) :
#				(min,max) pour uniforme, (moy,ecart-type) pour loi normale,
#				(moy du log, ecart-type du log) pour loi lognormale,
#				(forme,echelle) pour loi de Weibull, (lambda) pour loi
#				exponentielle, loi beta, loi triangulaire, loi trapezoidale
#				Par defaut, on a la loi uniforme avec (0,1)
#               indices1 et 2 : indique les indices (simples et doubles) devant etre
#                                       calcules; par defaut tous les indices simples
#               affichage : TRUE pour indiquer les resultats intermediaires
#-----------------------------------------------------------------------------------------------|
#Sortie : les indices eta a la fin du fichier "res-ENTROP.out" (fichier non ecrase)
#-----------------------------------------------------------------------------------------------|

#initialisation des tableaux contenant les indices
Si<-FALSE
for (i in 1:dim_x) 
	if (indices1[i]) Si<-TRUE
if (Si) HK<-matrix(nrow=R,ncol=dim_x)
Sij<-FALSE
for (i in 1:dim_x) 
	for (j in 1:dim_x)    
		if (indices2[i,j]) Sij<-TRUE
if (Sij) HK2<-array(dim=c(R,dim_x,dim_x))

#-------
#Boucle |
#-------

# on repete le programme R fois
for(z in 1:R) {

if (affichage) print(z)

#Tirage de l'echantillon
if (LHS) X<-samplingLHS(dim_x,S,lois,paramlois)
else if (det) X<-samplingDet(dim_x,S,paramlois)
else X<-samplingSimple(dim_x,S,lois,paramlois)
for(i in 1:dim_x)
	if (logx[i]) X[,i]<-log(X[,i])

#-------------------------------
#Calcul de la reponse Y |
#-------------------------------

if (methode>0) Y<-predictSR(modele,X,methode)
else Y<-f(X)

#transformation eventuelle si Y est de dimension 1
if (NCOL(Y)==1) {
	aux<-array(Y,dim=c(S,1))
	Y<-aux
}

for(i in 1:dim_y)
	if (logy[i]) Y[,i]<-exp(Y[,i])

#precalculs: bornes sur les echantillons et sur Y
infX<-vector(length=dim_x)
supX<-vector(length=dim_x)
for (i in 1:dim_x) infX[i]<-min(X[,i])
for (i in 1:dim_x) supX[i]<-max(X[,i])
infY<-vector(length=dim_y)
supY<-vector(length=dim_y)
for (i in 1:dim_y) infY[i]<-min(Y[,i])
for (i in 1:dim_y) supY[i]<-max(Y[,i])

#--------------------------------------------------
#Calcul de H0==H(Y) (si necessaire) |
#--------------------------------------------------

estimationsY<-rep(0,(n+1)^dim_y)

#precalcul: determination des pas pour chaque composante
pasY<-vector(length=dim_y)
#decoupage de l'intervalle de variation de chaque Y[,i] en n morceaux
pasY<-(supY-infY)/n

#precalcul: nombres en base n+1 = indices de estimationsY
casesY<-rep(0,S)

for (i in 1:S) 
	for (j in 1:dim_y) casesY[i]<-casesY[i]+(floor((Y[i,j]-infY[j])/pasY[j]))*(n+1)^(j-1);
casesY<-casesY+1

#precalcul: tableau de repartition spatiale de Y
for (i in 1:S) estimationsY[casesY[i]]<-estimationsY[casesY[i]]+1

#normalisation
estimationsY<-estimationsY/S

if (!rel) {
	tmp<-rep(0,(n+1)^dim_y)
	for (j in 1:(n+1)^dim_y)
		if (estimationsY[j]>0) tmp[j]<- -estimationsY[j]*log(estimationsY[j])  
	H0<-sum(tmp)
	if (affichage) {
		print("entropie:")
		print(H0)
	}
}

#--------------
#Calcul des Hi |
#--------------

casesX<-matrix(nrow=S,ncol=dim_x)

if (Si) {

	for(i in 1:dim_x)
		if (indices1[i]) {
			#initialisations
			estimationsX<-rep(0,n+1)
			if (Si) estimationsXY<-matrix(0,nrow=n+1,ncol=(n+1)^dim_y)
	
			#decoupage de l'intervalle de variation de X[,i] en n morceaux
			pasX<-(supX[i]-infX[i])/n
	
			#precalcul: tableau de repartition spatiale de X[,i], puis de (X[,i],Y)
			for (j in 1:S) {
				aux<-floor((X[j,i]-infX[i])/pasX)+1
				estimationsX[aux]<-estimationsX[aux]+1
				casesX[j,i]<-aux
				if (Si) estimationsXY[aux,casesY[j]]<-estimationsXY[aux,casesY[j]]+1
			}
   
			#normalisations
			estimationsX<-estimationsX/S
			estimationsXY<-estimationsXY/S

			tmp<-matrix(0,nrow=n,ncol=(n+1)^dim_y)
			# dans cet ordre, car les affectations sont plus rapides que les additions
			for (j in 1:n)
				for (k in 1:(n+1)^dim_y) {
					if (estimationsXY[j,k]>0)
						tmp[j,k]<-estimationsXY[j,k]*log(estimationsXY[j,k]/(estimationsX[j]*estimationsY[k]))
				}
			HK[z,i]<-sum(tmp)
		}
		else {
			for (j in 1:S) 
				casesX[j,i]<-floor((X[j,i]-infX[i])/pasX)+1
		}
}

else {
	for (i in 1:dim_x)
		for (j in 1:S)
			casesX[j,i]<-floor((X[j,i]-infX[i])/pasX)+1
}

#----------------
#Calcul des Hij |
#----------------

if (Sij) {

	for(i in 1:(dim_x-1)) {
		HK2[z,i,i]<-HK[z,i]
		for (j in (i+1):dim_x)
			if (indices2[i,j]) {
				#initialisations
				estimationsXX<-matrix(0,nrow=n+1,ncol=n+1)
				estimationsXXY<-array(0,dim=c(n+1,n+1,(n+1)^dim_y))

				# precalcul: tableau de repartition spatiale de (X[,i],X[,j],Y), et (X[,i],X[,j]):
				for (k in 1:S) {
					estimationsXX[casesX[k,i],casesX[k,j]]<-estimationsXX[casesX[k,i],casesX[k,j]]+1
					estimationsXXY[casesX[k,i],casesX[k,j],casesY[k]]<-estimationsXXY[casesX[k,i],casesX[k,j],casesY[k]]+1
				}

				#normalisation
				estimationsXX<-estimationsXX/S
				estimationsXXY<-estimationsXXY/S

				tmp<-array(0,dim=c(n,n,(n+1)^dim_y))
				for (k in 1:n)
					for (l in 1:n)
						for (m in 1:(n+1)^dim_y) {
							if (estimationsXXY[k,l,m]>0)
								tmp[k,l,m]<-estimationsXXY[k,l,m]*log(estimationsXXY[k,l,m]/(estimationsXX[k,l]*estimationsY[m]))
						}
				HK2[z,i,j]<-sum(tmp)
				HK2[z,j,i]<-HK2[z,i,j]
			}
	}
	HK2[z,dim_x,dim_x]<-HK[z,dim_x]

}

#-----------------------------
#Calcul des indices de Sobol |
#-----------------------------

if (rel) {
	if (Si) {
		aux<-sum(HK[z,])
		HK[z,]<-HK[z,]/aux
	}
	if (Sij) {
		aux<-sum(HK2[z,,])
		HK2[z,,]<-HK2[z,,]/aux
	}
}
else {
	if (Si) HK[z,]<-HK[z,]/H0
	if (Sij) HK2[z,,]<-HK2[z,,]/H0
}

if (Si && affichage) {
	print("indices1:")
	print(HK[z,])
}

if (Sij && affichage) {
	print("indices2:")
	print(HK2[z,,])
}

#------------------
#Fin de la boucle |
#------------------

}

#--------------------------------------------
#Calcul des moyennes, min, max et ecart-types |
#--------------------------------------------

#Initialisation des vecteurs

if (Si) {
	mS1<-vector(length=dim_x)
	minS1<-vector(length=dim_x)
	maxS1<-vector(length=dim_x)
	sigS1<-vector(length=dim_x)
}

if (Sij) {
	mS2<-matrix(nrow=dim_x,ncol=dim_x)
	minS2<-matrix(nrow=dim_x,ncol=dim_x)
	maxS2<-matrix(nrow=dim_x,ncol=dim_x)
	sigS2<-matrix(nrow=dim_x,ncol=dim_x)
}

#Calculs

if (Si) {

	for(i in 1:dim_x)
		if (indices1[i]) {
			mS1[i]<-mean(HK[,i])
			minS1[i]<-min(HK[,i])
			maxS1[i]<-max(HK[,i])
			sigS1[i]<-sqrt(var(HK[,i]))
		}
}

if (Sij) {

	for(i in 1:(dim_x-1))
		for(j in (i+1):dim_x)
			if (indices2[i,j]) {
				mS2[i,j]<-mean(HK2[,i,j])
				minS2[i]<-min(HK2[,i,j])
				maxS2[i]<-max(HK2[,i,j])
				sigS2[i]<-sqrt(var(HK2[,i,j]))
			}
}

#-----------------------
#Affichage dans fichier |
#-----------------------

#Preparation des donnees pour l'affichage

if (Si) {

mS1<-format(mS1,digits=3)
minS1<-format(minS1,digits=3)
maxS1<-format(maxS1,digits=3)
sigS1<-format(sigS1,digits=3)

}

if (Sij) {

mS2<-format(mS2,digits=3)
minS2<-format(minS2,digits=3)
maxS2<-format(maxS2,digits=3)
sigS2<-format(sigS2,digits=3)

}

FSOB <- file("res-SOBOL.out","a+")
#FSOB <- file("D:/res-SOBOL.out","a+")
#FSOB <- file("/home/iooss/benjamin/res-ENTROP.out","a+")
write("--------------------------------------------------",file=FSOB)

if (methode==1){
	write("Modele GLM : Indices de sensibilite",file=FSOB)
	print("Entrop GLM termine");}
if (methode==2){
	write("Modele PLS : Indices de sensibilite",file=FSOB)
	print("Entrop PLS termine");}
if (methode==3){
	write("Modele SVM : Indices de sensibilite",file=FSOB)
	print("Entrop SVM termine");}
if (methode==4){
	write("Modele BOOSTING : Indices de sensibilite",file=FSOB)
	print("Entrop BOOSTING termine");}
if (methode==5){
	write("Modele FORETS ALEATOIRES : Indices de sensibilite",file=FSOB)
	print("Entrop FORETS ALEATOIRES termine");}
if (methode==6){
	write("Modele RESEAU de NEURONES : Indices de sensibilite",file=FSOB)
	print("Entrop RESEAU de NEURONES termine");}

write(" ",file=FSOB)

if (Si) {

	write("------------------",file=FSOB)
	write("| Les indices eta_i |",file=FSOB)
	write("------------------",file=FSOB)

	for(i in 1:dim_x)
		if (indices1[i]) {
			if (R>1) write(c("eta_",i," : Moy=",mS1[i]," Min=",minS1[i]," Max=",maxS1[i]," E-t=",sigS1[i]),ncolumns=10,file=FSOB)
			else write(c("eta_",i,"=",mS1[i]),ncolumns=10,file=FSOB)
		}
	write(" ",file=FSOB)
}

if (Sij) {

	write("-------------------",file=FSOB)
	write("| Les indices eta_ij |",file=FSOB)
	write("-------------------",file=FSOB)

	for(i in 1:(dim_x-1))
		for(j in (i+1):dim_x)
			if (indices2[i,j]) {
				if (R>1) write(c("eta_",i,j," : Moy=",mS2[i,j]," Min=",minS2[i]," Max=",maxS2[i]," E-t=",sigS2[i]),ncolumns=11,file=FSOB)
				else write(c("eta_",i,j,"=",mS2[i,j]),ncolumns=11,file=FSOB)
			}
	write(" ",file=FSOB)
}

close(FSOB)
}
