#appel: sobol(f=hs,dim_x=3,S=10000,R=1,LHS=TRUE,paramlois=array(c(-pi,pi,-pi,pi,-pi,pi),dim=c(2,3)))

sobol<-function(f=a,modele=b,methode=0,dim_x,S=10000,R=100,logx=rep(FALSE,dim_x),logy=FALSE,
	LHS=FALSE,lois=rep(0,dim_x),paramlois=array(0,dim=c(2,dim_x)),
	indices1=rep(TRUE,dim_x),indices2=matrix(FALSE,nrow=dim_x,ncol=dim_x),
	indicest=rep(FALSE,dim_x),affichage=FALSE) {

#---------------------------------------------------------------------------|
#Fonction pour calculer les indices de sensibilite de Sobol                 |
#---------------------------------------------------------------------------|
#Arguments obligatoires dans le cas d'un modifie :
#	modele : objet contenant le modele
#	methode : numero de la methode, 0 si fonction utilisateur
#	dim_x : nombre de parametres d'entree dans le modele
#
#Arguments obligatoires dans le cas d'une fonction :
#	f : objet representant la fonction (on extrait ensuite ses parametres..etc)
#	dim_x : nombre de parametres d'entree dans le modele
#
#Arguments facultatifs :
#	S : nombre de simulations
#	R : nombre de repetitions du programme
#	logx : vecteur logx=c(TRUE,TRUE,...,TRUE), correspondant aux
#		parametres d'entree transformees en log lors de
#		l'ajustement du modele
#	logy : mettre logy=TRUE si la variable de sortie a ete
#		transformee en log lors de l'ajustement du modele
#	LHS : TRUE si on veut un echantillonage par hypercube latin
#	lois : vecteur contenant les types de distribution de proba
#		pour chaque Vi
#		0=uniforme ; 1=normale ; 2=lognormale ; 3=weibull
#		4=exponentielle; loi beta, loi triangulaire, loi trapezoidale
#		par defaut: loi uniforme
#	paramlois : tableau contenant les parametres de chaque loi
#		pour chaque Vi (range par colonne) :
#		(min,max) pour uniforme, (moy,ecart-type) pour loi normale,
#		(moy du log, ecart-type du log) pour loi lognormale,
#		(forme,echelle) pour loi de Weibull, (lambda) pour loi
#		exponentielle, (n, lambda) pour loi gamma
#		Par defaut: loi uniforme sur [0,1]
#---------------------------------------------------------------------------|
#Sortie : les indices Si, Sij et STi a la fin du fichier "res-SOBOL.out"    
#         (fichier non ecrase)                                              
#---------------------------------------------------------------------------|

#Initialisation des vecteurs contenant les indices de sensibilite
Si<-FALSE
for (i in 1:dim_x) 
	if (indices1[i]) Si<-TRUE
if (Si) S1<-matrix(nrow=R,ncol=dim_x)
Sij<-FALSE
for (i in 1:dim_x) 
	for (j in 1:dim_x)    
		if (indices2[i,j]) Sij<-TRUE
if (Sij) S2<-array(dim=c(R,dim_x,dim_x))
Sti<-FALSE
for (i in 1:dim_x) 
	if (indicest[i]) Sti<-TRUE
if (Sti) St<-matrix(nrow=R,ncol=dim_x)

#--------
#Boucle |
#--------

for(k in 1:R) {

if (affichage) print(k)

#Tirage des echantillons
	if (!LHS) {
		X1<-samplingSimple(dim_x,S,lois,paramlois)
		X2<-samplingSimple(dim_x,S,lois,paramlois)
	}
	else{
		X1<-samplingLHS(dim_x,S,lois,paramlois)
		X2<-samplingLHS(dim_x,S,lois,paramlois)
	}
for(i in 1:dim_x)
	if (logx[i]) {
		X1[,i]=log(X1[,i])
		X2[,i]=log(X2[,i])
	}

#---------------------------
#Calcul de la moyenne de y |
#---------------------------

if (methode>0) pred<-predictSR(modele,X1,methode)
else pred<-f(X1)
if (logy) pred=exp(pred)
my<-mean(pred)

#-------------
#Calcul de V0 |
#-------------

V0<-var(pred)
if (affichage) {
	print("variance:")
	print(V0)
}

#---------------
#Calcul des Vi |
#---------------

if (Si) {

	V<-rep(0,dim_x)

	for(i in 1:dim_x)
		if (indices1[i]) {
			sav<-X2[,i]
			X2[,i]<-X1[,i]
			if (methode>0) pred2<-predictSR(modele,X2,methode)
			else pred2<-f(X2)
			if (logy) pred2=exp(pred2)
			S1[k,i]<-mean(pred*pred2)-my^2
			X2[,i]<-sav
	}
}

#----------------
#Calcul des Vij |
#----------------

if (Sij) {

	V2<-matrix(0,nr=dim_x,nc=dim_x)

	for(i in 1:(dim_x-1))
		for(j in (i+1):dim_x) 
			if (indices2[i,j]) {
				savi<-X2[,i]
				savj<-X2[,j]
				X2[,i]<-X1[,i]
				X2[,j]<-X1[,j]
				if (methode>0) pred2<-predictSR(modele,X2,methode)
				else pred2<-f(X2)
				if (logy) pred2=exp(pred2)
				S2[k,i,j]<-mean(pred*pred2)-V[i]-V[j]-my^2
				X2[,i]<-savi[,i]
				X2[,j]<-savj[,j]
			}
}

#----------------
#Calcul des Vti |
#----------------

if (Sti) {

	Vt<-rep(0,dim_x)
	X1b<-X1

	for(i in 1:dim_x)
		if (indicest[i]) {
			X1b[,i]<-X2[,i]
			if (methode>0) pred2<-predictSR(modele,X1b,methode)
			else pred2<-f(X1b)
			if (logy) pred2=exp(pred2)
			St[k,i]<-mean(pred*pred2)-my^2
			X1b[,i]<-X1[,i]
		}
}

#-----------------------------
#Calcul des indices de Sobol |
#-----------------------------

if (Si) S1[k,]<-S1[k,]/V0
if (Sij) S2[k,,]<-S2[k,,]/V0
if (Sti) St[k,]<-1-St[k,]/V0

if (affichage) {
	if (Si) {
		print("indices1:")
		print(S1[k,])
	}
	if (Sij) {
		print("indices2:")
		print(S2[k,,])
	}
	if (Sti) {
		print("indicest:")
		print(St[k,])
	}
}

#------------------
#Fin de la boucle |
#------------------

}

#--------------------------------------------
#Calcul des moyennes, min, max et ecart-types |
#--------------------------------------------

#Initialisation des vecteurs

if (Si) {

	mS1<-vector(length=dim_x)
	minS1<-vector(length=dim_x)
	maxS1<-vector(length=dim_x)
	sigS1<-vector(length=dim_x)
}

if (Sij) {

	mS2<-matrix(nrow=dim_x,ncol=dim_x)
	minS2<-matrix(nrow=dim_x,ncol=dim_x)
	maxS2<-matrix(nrow=dim_x,ncol=dim_x)
	sigS2<-matrix(nrow=dim_x,ncol=dim_x)
}

if (Sti) {

	mSt<-vector(length=dim_x)
	minSt<-vector(length=dim_x)
	maxSt<-vector(length=dim_x)
	sigSt<-vector(length=dim_x)
}

#Calculs

if (Si) {

	for(i in 1:dim_x){
		mS1[i]<-mean(S1[,i])
		minS1[i]<-min(S1[,i])
		maxS1[i]<-max(S1[,i])
		sigS1[i]<-sqrt(var(S1[,i]))
	}
}

if (Sij) {

	for(i in 1:(dim_x-1))
		for(j in (i+1):dim_x) {
			mS2[i,j]<-mean(S2[,i,j])
			minS2[i]<-min(S2[,i,j])
			maxS2[i]<-max(S2[,i,j])
			sigS2[i]<-sqrt(var(S2[,i,j]))
		}
}

if (Sti) {

	for(i in 1:dim_x) {
		mSt[i]<-mean(St[,i])
		minSt[i]<-min(St[,i])
		maxSt[i]<-max(St[,i])
		sigSt[i]<-sqrt(var(St[,i]))
	}
}

#-----------------------
#Affichage dans fichier |
#-----------------------

#Preparation des donnees pour l'affichage

if (Si) {

	mS1<-format(mS1,digits=3)
	minS1<-format(minS1,digits=3)
	maxS1<-format(maxS1,digits=3)
	sigS1<-format(sigS1,digits=3)
}

if (Sij) {

	mS2<-format(mS2,digits=3)
	minS2<-format(minS2,digits=3)
	maxS2<-format(maxS2,digits=3)
	sigS2<-format(sigS2,digits=3)
}

if (Sti) {

	mSt<-format(mSt,digits=3)
	minSt<-format(minSt,digits=3)
	maxSt<-format(maxSt,digits=3)
	sigSt<-format(sigSt,digits=3)
}

FSOB <- file("res-SOBOL.out","a+")
#FSOB <- file("D:/res-SOBOL.out","a+")
#FSOB <- file("/home/iooss/benjamin/res-SOBOL.out","a+")

write("--------------------------------------------------",file=FSOB)

if (methode==1) {
	write("Modele GLM : Indices de sensibilite",file=FSOB)
	print("Sobol GLM termine");}
if (methode==2) {
	write("Modele PLS : Indices de sensibilite",file=FSOB)
	print("Sobol PLS termine");}
if (methode==3) {
	write("Modele SVM : Indices de sensibilite",file=FSOB)
	print("Sobol SVM termine");}
if (methode==4) {
	write("Modele BOOSTING : Indices de sensibilite",file=FSOB)
	print("Sobol BOOSTING termine");}
if (methode==5) {
	write("Modele FORETS ALEATOIRES : Indices de sensibilite",file=FSOB)
	print("Sobol FORETS ALEATOIRES termine");}
if (methode==6) {
	write("Modele RESEAU de NEURONES : Indices de sensibilite",file=FSOB)
	print("Sobol RESEAU de NEURONES termine");}

write(" ",file=FSOB)

if (Si) {

	write("------------------",file=FSOB)
	write("| Les indices Si |",file=FSOB)
	write("------------------",file=FSOB)

	for(i in 1:dim_x)
		if (indices1[i])
			write(c("S",i," : Moy=",mS1[i]," Min=",minS1[i]," Max=",maxS1[i]," E-t=",sigS1[i]),ncolumns=10,file=FSOB)
	write(" ",file=FSOB)
}

if (Sij) {

	write("-------------------",file=FSOB)
	write("| Les indices Sij |",file=FSOB)
	write("-------------------",file=FSOB)

	for(i in 1:(dim_x-1))
		for(j in (i+1):dim_x)
			if (indices2[i,j])
				write(c("S",i,j," : Moy=",mS2[i,j]," Min=",minS2[i]," Max=",maxS2[i]," E-t=",sigS2[i]),ncolumns=11,file=FSOB)
	write(" ",file=FSOB)
}

if (Sti) {

	write("-------------------",file=FSOB)
	write("| Les indices Sti |",file=FSOB)
	write("-------------------",file=FSOB)

	for(i in 1:dim_x) 
		if (indicest[i])
			write(c("St",i," : Moy=",mSt[i]," Min=",minSt[i]," Max=",maxSt[i]," E-t=",sigSt[i]),ncolumns=10,file=FSOB)
	write(" ",file=FSOB)
}

close(FSOB)
}

